@@ -301,7 +301,12 @@ get_oom_object (LPDISPATCH pStart, const char *fullname)
 
         if (n >= sizeof name)
           n = sizeof name - 1;
+/* As you can see above n is checked for bounds of name. But
+   GCC warns because it just looked at n = strlen (fullname). */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wstringop-truncation"
         strncpy (name, fullname, n);
+#pragma GCC diagnostic pop
         name[n] = 0;
 
         if (dot)
@@ -3004,16 +3009,19 @@ log_addins ()
           continue;
         }
 
-      char *progId = get_oom_string (addin, "ProgId");
+      std::string progId = get_oom_string_s (addin, "ProgId");
+      std::string description = get_oom_string_s (addin, "Description");
       gpgol_release (addin);
 
-      if (!progId)
+      if (progId.empty ())
         {
-          TRACEPOINT;
-          continue;
+          progId = "Unknown ProgID";
         }
-      activeAddins += std::string (progId) + "\n";
-      xfree (progId);
+      if (description.empty ())
+        {
+          description = "No description";
+        }
+      activeAddins += progId + " (" + description + ")"  + "\n";
     }
   gpgol_release (addins);
 
